/* CUSUM Procedures
**
** CUSUM(resids)        CUSUM procedure
** CUMSUMSQ(resids)     CUSUMSQ procedure
**
** Patrick T. Brandt
** Date: August 29, 1998
*/


/* CUSUM -- procedure to compute the standardized cummulative sum of a
**          vector
**
**  USAGE:  s = CUSUM(resids,gr,a);
**
**  INPUT:  resids = TxK vector of residuals
**              gr = graphing code; 0 = no graph, 1 = graph
**               a = significance level for graph; 0.05 or 0.1
**  OUTPUT:      s = TxK vector of standardized CUSUM values
**           and a graph, if gr=1
*/

proc cusum(resid,gr,a);
    local v,csm,ciu,cil;
v = (stdc(resid)^2)';
csm = ((cumsumc(resid)./v));

/* Branching to test values for graphs */
if gr/=0 AND gr/=1;
    "ERROR: CUSUM graphing code invalid; must be 0 or 1"; end;
    endif;

if gr==0;
    retp(csm);
elseif gr==1;
    ylabel("CUSUM Residuals");
    xlabel("Observation");

    if a/=0.1 AND a/=0.05;
        "ERROR: CUSUM significance level can't be computed";
        ciu = 0;
        cil = 0;
        xy(rows(resid),csm); retp(csm);
    else;
    ciu = cusumci(rows(resid),a);
    cil = -ciu;
    xy(rows(resid),csm~ciu~cil); retp(csm);
    endif;
endif;
endp;


/* CUSUMSQ -- procedure to compute the cummulative sum of squares vector
**
**  USAGE:  s = CUSUMSQ(resids);
**
**  INPUT:  resids = TxK vector of residuals
**  OUTPUT:      s = TxK vector of standardized CUSUMSQ values
**
*/

proc cumsumsq(resid);
    local v,ss;
ss = cumsumc(resid^2);
 v = sumc(resid^2)';
retp(ss./v);
endp;

proc (1)=cusumci(t,a);
    local ci,tt;
tt = seqa(1,1,t);
retp((a.*sqrt(t)) + ((2.*a.*tt)./sqrt(t)));
endp;
