/* HILLU - Hildreth-Lu procedure for estimating AR(1)
**         error OLS regressions
**
** Usage: {b,rho,cov} = hillu(y,x)
**
** INPUTS : Y = Tx1 vector dependent variable
**          X = TxK matrix of regressors
**
** OUTPUTS: b = estimated coefficients for mean
**          rho = estimated AR(1) coefficient
**          cov = covariance matrix computed from b and rho.  The
**                last row/column of cov are the covariances for rho.
**
** Remarks: This procedure uses a grid search to find the
**          value of rho.  It uses a fine grid (-0.999,0.999) to
**          compute the AR coefficient.  Then it computes the
**          model coefficients and the covariance matrix.
**          The code assumes that X does contain an intercept.
**          The code assumes that there are no lagged dependent variables.
**          See Davidson and MacKinnon (1993: 331-341) for details.
**
** August 16, 1998
**
** Patrick T. Brandt
*/

proc (3) = hillu(y,x);
    local ssr,j,xlag,xt,ylag,yt,yp,xp,rho,rhohat,cov,
            b,b1,e2,n,ssrmin,ssrsort,e,u,t,p;

@ set up lagged data @
n = rows(x);
ylag = trimr(lag1(y),1,0);
xlag = (trimr(lag1(x),1,0));
yt = trimr(y,1,0);     xt = trimr(x,1,0);
print "HILDRETH-LU : Computing optimal rho using grid search";
@ do loop for grid search @
ssr = 0;
rho = -0.999;
do while rho<1;
@ compute betas @
yp = yt-(rho.*ylag);
xp = xt-(rho.*xlag);
b1 = (inv(moment(xp,0)))*xp'*yp;     @ return the Kx1 b(rho)         @
e2 = (yp-(xp*b1))'*(yp-(xp*b1));     @ compute SSR                   @
if rho == -0.999;                    @ build matrix of ssr's and rho @
ssr = rho~e2;
else;
ssr = ssr|(rho~e2);
endif;
rho = rho + 0.001;
endo;

print "HILDRETH-LU : Computing final estimates";
@ now find the rho that minimizes ssr @
ssrsort = sortc(ssr,2);
rhohat = ssrsort[1,1];
yp = yt-(rhohat.*ylag);
xp = xt-(rhohat.*xlag);
b  = inv(moment(xp,0))*xp'*yp;       @ return the Kx1 b(rhohat) @
e = yp - xp*b;
cov = (1./(n-cols(x)-2))*(e'*e).*inv(moment(xp~(ylag-xlag*b),0));

/* if X contains lagged values, uncomment this section for robust est. */
/* u = ylag-xlag*b;
cov = (1./(n-cols(x)-1)*(e'*e).*inv(xp'*(u*(inv(u'*u))*u)*xp))
        |zeros(1,cols(x)+1)~(zeros(cols(x)+2,1));
cov[cols(x)+2,cols(x)+2] = (1-rhohat^2)./(n-1);
*/

/* print out section */

t = (b|rhohat)./(sqrt(diag(cov)));
p = (cdftc(abs(t),n-cols(x)-1));

format 8,4;
print "Hildreth-Lu model estimates";
print "-----------------------------------------------------------------";
print "Beta     S.E.'s      T-stat      p-value";
print (b|rhohat)~sqrt(diag(cov))~t~p;
print "-----------------------------------------------------------------";
print "Note: Last coefficient is rho";
print " ";
print "Number of cases            ";;n;
format 8,6;
print "Total Sum of Squares       ";;yt'*yt;
print "Sum of Squared Error       ";;e'*e;
print "Regression Sum of Squares  ";;(yt'*yt)-(e'*e);
print "Degrees of freedom         ";;n-cols(x)-1;
print "-----------------------------------------------------------------";

retp(b,rhohat,cov);
endp;

