/* LNGM - Log gamma function
**
** Purpose: computes the log gamma function
**
** Usage: y = lngm(x)
**
** Inputs: Nx1 vector, x>0
**
** Output: y, Nx1 vector of log gamma values
**
** Comments: Computes log gamma function based on the algorithm
**           in Press et al, 1986.
**
** Cites: Press et al. 1986, "Numerical Recipies"
**        Lanczos, C.  1964, Journal SIAM Numerical Analysis,
**        Ser B, Vol 1, p.86.
**
** Date: July 13, 1998
*/
proc (1)=lngm(x);
    local c,yp,ys,brkpnt,mask;
/* traps */
brkpnt = 1;         @ x below this, use direct computation @
if not (x >= 0);    @ some negative x's @
       "ERROR: Arguments must all be non-negative."; end;
endif;
brkpnt = brkpnt + 1;
    if (x > brkpnt);                /* use press approx. */
       gosub press; retp( ys );
    elseif (x <= brkpnt);           /* use ln(gamma) */
       gosub lngamma; retp( yp );
    else;                           /* some x's of each type */
       gosub press;
       gosub lngamma;
    mask = (x .> brkpnt);
    retp( ys.*mask + yp.*(.not mask) );
    endif;

/* subroutines */
    press:                          /* Press et al. algorithm */
let c[7,1] = { 1, 76.18009173, -86.50532033, 24.01409822,
        -1.231739516, 0.120858003e-2, -0.536382e-5 };
      ys = ((x-0.5).*ln(x + 4.5)) - (x + 4.5)
    + ln(sqrt(2.*pi).*(c[1,1] + c[2,1]./x + c[3,1]./(x+1)
         + c[4,1]./(x+2) + c[5,1]./(x+3) + c[6,1]./(x+4)
         + c[7,1]./(x+5)));
    return;

    lngamma:
      yp = ln( gamma(x) );
    return;
endp;

