/* PARP -- Poisson AR(p) state-space model
**
** Purpose: Estimates a time series model for count data.  The model is
**          based on an extended Kalman filter where the state
**          variable follows an AR(p).
**
** Usage: {rho,d,cov,llf,aic} = parp(y,x,p,sv)
**
** Inputs:
**      y = Tx1 vector of counts
**      x = TxK matrix of covariates
**      p = order of autoregression
**     sv = starting value code or starting values
**
** If the user provides the starting values, they must be in a ((K+P)x1) vector
** in the form,
**
**  b1|b2|...|bk|r1|r2|...|rp,
**
** where
** bj is the j'th regression coefficient
** rk is the k'th AR coefficient
**
** Starting value codes:
**
**  sv = "P"; Lagged Poisson regression
**  sv = "OLS"; OLS on the natural logarithm of y
**  sv = vector; a vector of starting values
**
** Outputs:
**        rho = Px1 vector of AR coefficients
**          d = Kx1 matrix of regression parameters
**        cov = (K+P)x(K+P) covariance matrix
**        llf = final log-likelihood value
**        aic = Akaike Information Criterion value
**
** Notes: This code assumes you have a version of Gauss' MAXLIK module.
**    It is written to run under MAXLIK versions 3 and 4.  See XXX
**    in the supplied documentation for some helpful hints on using
**    MAXLIK with this code (including the section "Optimization
**    Algorithms").  Thus, all maxlik options are available in this
**    procedure, including the naming of the variables for the
**    MAXLIK output.
**
** Disclaimer: This code is copyright, Patrick T. Brandt 1998.
** Any use is at you own risk, and this code is not for commercial
** purposes.  If you have any question that cannot be answered in the
** supplied documentation, e-mail pbrandt@indiana.edu.
**
** Date: July 13, 1998
**
*/

proc (5)=parp(y,x,p,sv);
    local l,rho,d,cov,llf,aic,startv,blols,parest,f,g,ret,x1,ly;

/* SET-UP of Data and error checking */

if (y<0);
    errorlog "ERROR: PARP - Some counts are negative";
    retp(error(99));
endif;

if y[1,1]==0;
    errorlog "ERROR: PARP - First observation cannot be zero";
    retp(error(99));
endif;

/* Estimate starting values */
print "Computing starting values by ";;
l=1;        @ set up the lags @
ly = 0;
do until l>p;
    if l==1;
    ly = lag1(y);
    else;
    ly = ly~lag1(ly[.,l-1]);
    endif;
l = l+1;
endo;

x1 = trimr(x~(ly),p,0);
blols = inv(x1'*x1)*x1'*trimr(ln(y+0.01),p,0);

if type(sv).==13;                   @ flag whether sv is a string @
if sv.=="P";
print "Lagged Poisson regression.";
__output = 0;
{startv,f,g,cov,ret} = maxlik(trimr(y,p,0)~x1,0,&lpsn,blols');

elseif sv.=="OLS";
print "Lagged Logged OLS.";
startv = blols;
endif;

elseif type(sv).==6;
print "User Supplied Starting Values";
startv = sv;
endif;

/* Call the likelihood procedure */
__output = 2;
__title = "Poisson AR(p) Regression Estimates";
{parest,llf,g,cov,ret} = maxlik(y~x,0,&lparp,startv);

/* Print results and write output matrices */

call maxprt(parest,llf,g,cov,ret);
print " ";
print "------------------------------------------------------------";
print " ";
{aic,llf} = aicllf(y~x,parest,&lparp);
print " ";
print "------------------------------------------------------------";
print " ";

rho = parest[cols(x)+1:cols(x)+p,1];

if cols(x)==1;
d = parest[1,1];
else;
d = parest[1:cols(x),1];
endif;

retp(rho,d,cov,llf,aic);
endp;

