/* PEWMA -- Poisson Exponentially Weighted Moving Average Model
**
** Purpose: Estimates a local level time series model for count data.
**          The model is based on an extended Kalman filter where
**          the state variable follows an EWMA.
**
** Usage: {omega,d,cov,llf,aic} = pewma(y,x,sv)
**
** Inputs:
**      y = Tx1 vector of counts
**      x = TxK matrix of covariates
**     sv = starting value code or starting values
**
** If the user provides the starting values, they must be in a ((K+1)x1) vector
** in the form,
**
**  omega|b2|...|bk,
**
** where
** omega is EWMA parameter
** bj    is the j'th regression coefficient
**
** Starting value codes:
**
**  sv = "P"; Poisson regression (default); initial value of w = 1;
**  sv = "OLS"; OLS on the natural logarithm of y; inital value of w = 1;
**  sv = "EWMA"; Gaussian EWMA; inital value of w estimated by EWMA;
**  sv = vector; a vector of starting values.
**
** Outputs:
**      omega = scalar estimate of hyperparameter for EWMA
**          d = Kx1 matrix of regression parameters
**        cov = (K+1)x(K+1) covariance matrix
**        llf = final log-likelihood value
**        aic = Akaike Information Criterion value
**
** Notes: This code assumes you have a version of Gauss' MAXLIK module.
**    It is written to run under MAXLIK versions 3 and 4.  See
**    the supplied documentation for some helpful hints on using
**    MAXLIK with this code (including the section "Optimization
**    Algorithms").  Thus, all maxlik options are available in this
**    procedure, including the naming of the variables for the
**    MAXLIK output.
**
** This code is copyright, Patrick T. Brandt 1998.
** Any use is at you own risk, and this code is not for commercial
** purposes.  If you have any question that cannot be answered in the
** supplied documentation, e-mail pbrandt@indiana.edu.
**
** Date: August 15, 1998
**
*/


proc (5)=pewma(y,x,sv);
    local omega,d,cov,llf,aic,startv,stvw,bpsn,blols,pewest,f,g,ret,ly,x1;

/* SET-UP of Data and error checking */

if (y<0);
    errorlog "ERROR: PEWMA - Some counts are negative";
    retp(error(99));
endif;

if y[1,1]==0;
    errorlog "ERROR: PEWMA - First observation cannot be zero";
    retp(error(99));
endif;

/* Estimate starting values */
x1 = ones(rows(y),1)~x;
blols = invpd(x1'*x1)*x1'*ln(y+0.01);
print "Computing starting values by ";;

if type(sv).==13;                   @ flag whether sv is a string @
  if sv.=="P";
    print "Poisson regression.";
    __output = 0;
    {startv,f,g,cov,ret} = maxlik(y~x1,0,&lpsn,blols');
    startv = 1|trimr(startv,1,0);
  elseif sv.=="OLS";
    print "Logged OLS.";
    startv = 1|trimr(blols,1,0);
  elseif sv.=="EWMA";
    print "EWMA estimator (This may take a while!)";
    stvw = ewmaest(ln(y+0.01));
    ly = ln(y+0.01)-ewmav(ln(y+0.01),stvw);
    startv = stvw|invpd(x'*x)*x'*ly;
  endif;
elseif type(sv).==6;
print "User Supplied Starting Values";
startv = sv;
endif;

/* Call the likelihood procedure */
__output = 2;
__title = "PEWMA regression estimates";
{pewest,llf,g,cov,ret} = maxlik(y~x,0,&lpg,startv);

/* Print results and write output matrices */

call maxprt(pewest,llf,g,cov,ret);
print " ";
print "------------------------------------------------------------";
print " ";
print "T-test for Poisson versus PEWMA model";
print "H0: omega = 1";
print "HA: omega < 1";
print "------------------------------------------------------------";
print " ";
print "Test value       ";;(1-pewest[1,1])./(sqrt(cov[1,1]));
print "P-value for test ";;
       cdftc(abs((1-pewest[1,1])./(sqrt(cov[1,1]))),rows(y)-cols(x)-1);
print " ";
print "------------------------------------------------------------";
print " ";
{aic,llf} = aicllf(y~x,pewest,&lpg);
print " ";
print "------------------------------------------------------------";
print " ";

omega = pewest[1,1];

if cols(x)==1;
d = pewest[2,1];
else;
d = pewest[2:cols(x)+1,1];
endif;

retp(omega,d,cov,llf,aic);
endp;
