/* POISSON -- A procedure to compute Poisson regressions
**
** Purpose: Estimates a Poisson regression.
**
** Usage: {d,cov,llf,aic} = poisson(y,x,sv)
**
** Inputs:
**      y = Tx1 vector of counts
**      x = TxK matrix of covariates
**     sv = scalar, starting value code
**
** Starting value codes:
**
**  sv = 0; OLS on the natural logarithm of y;
**  sv = 1; Zeros;
**
** Outputs:
**          d = Kx1 matrix of regression parameters
**        cov = (K)x(K) covariance matrix
**        llf = final log-likelihood value
**        aic = Akaike Information Criterion value
**
** Notes: This code assumes you have a version of Gauss' MAXLIK module.
**    It is written to run under MAXLIK versions 3 and 4.  See XXX
**    in the supplied documentation for some helpful hints on using
**    MAXLIK with this code (including the section "Optimization
**    Algorithms").  Thus, all maxlik options are available in this
**    procedure, including the naming of the variables for the
**    MAXLIK output.
**
** This code is copyright, Patrick T. Brandt 1998.
** Any use is at you own risk, and this code is not for commercial
** purposes.  If you have any question that cannot be answered in the
** supplied documentation, e-mail pbrandt@indiana.edu.
**
** Date: August 15, 1998
**
*/

/* include the required procedures */

#include lpsn.g;
#include aicllf.g;

proc (4)=poisson(y,x,sv);
    local d,cov,llf,aic,blols,f,g,ret;

/* SET-UP of Data and error checking */

if (y<0);
    errorlog "ERROR: POISSON - Some counts are negative";
    retp(error(99));
endif;

/* Estimate starting values */
if sv==0;
blols = invpd(x'*x)*x'*ln(y+0.01);
elseif sv==1;
blols = zeros(1,cols(x));
endif;
/* Call the likelihood procedure */
__output = 2;
__title = "Poisson regression estimates";
{d,f,g,cov,ret} = maxlik(y~x,0,&lpsn,blols');

/* Print results and write output matrices */

call maxprt(d,f,g,cov,ret);
print " ";
print "------------------------------------------------------------";
print " ";
{aic,llf} = aicllf(y~x,d,&lpsn);
print " ";
print "------------------------------------------------------------";
print " ";

retp(d,cov,llf,aic);
endp;

