/* PEWMADGP Poisson Exponentially Weighted Moving Average simulator
**
** Purpose: Generates PEWMA data.
**          Data follows a state-space model with:
**
**              y(t)~Po(mu(t))
**
**              mu(t)~G(a,b)
**
**
** USAGE: {y,mu} = pewmadgp(n,a0,b0,x,w,d);
**
**      y = Poisson distributed vector
**     mu = conditional mean used to generate each count
**      n = length of sample path
**      a0 = prior for a
**      b0 = prior for b
**       x = vector of covariates (nxk--without a constant)
**       w = weighting parameter for EWMA of mu
**       d = kx1 vector of parameters for link function
**
**/
#include psi.g;
#include rndp.g;
#include betarng.g;
#include ewma.g;

proc (2)=pewmadgp(n,a0,b0,x,w,d);
    local t,j,mu,y,at1,bt1,btt1,att1,r,a,b,m,eta;

/* initialize matrices */
y = zeros(n,1);
mu = zeros(n,1); r = zeros(n,1);
eta = zeros(n,1);
t = 1;
do until t>n;
if t==1;
  r[t,1] = psi(a0) - psi(a0.*w);
  m = (a0.*w.*exp(r[t,1]))./(b0.*w.*exp(-x[t,.]*d));
  y[t,1] = rndp(1,1,m);
@ compute the posterior for the next round's prior @
  a = a0.*w + y[t,1];   b = b0.*w + exp(x[t,.]*d + r[t,1]);
  mu[t,1] = 0 + y[t,1];

else;

@ compute the conditional values @
    r[t,1] = psi(a) - psi(a.*w);
    att1 = a.*w;
    btt1 = b.*w.*exp(-x[t,.]*d - r[t,1]);
@ filter through transition eqn @
    eta[t,1] = betarng(a.*w,(1-w).*a,1);
    m = exp(x[t,.]*d + r[t,1]).*
        ewma(y[1:t-1,1],w)./ewma(exp(x[1:t-1,.]*d + r[1:t-1,1]),w);
@ generate observation @
    y[t,1] = rndp(1,1,m);
@ compute posterior/next prior @
    a = att1 + y[t,1];   b = w.*b + exp(x[t,.]*d + r[t,1]);
    mu[t,1] = exp(x[t,.]*d + r[t,1]).*a./b;
endif;
t = t+1;
endo;
retp(y,mu);
endp;

