/* PEWMA_FILTER - A Filtering and Smoothing Algorithm for Poisson DGLM
**
** Purpose: The procedure computes a filter for PEWMA data Based on Harvey and Fernandes
**          1989 (JBES) and Shephard (1994).  See Brandt and Williams (1998) for details.
**
** USAGE: {a,aa,aaa,b,bb,bbb,mu,r} = pewma_filter(y,x,w,d)
**
** INPUTS:  y = Tx1 vector of series to be filtered
**          x = TxK matrix of covariates
**          w = EWMA parameter
**          d = Kx1 vector of covariate parameters
**
** OUTPUTS:  a = vector of a(t-1) parameters
**          aa = vector of a(t|t-1) parameters
**         aaa = vector of a(t) parameters
**           b = vector of b(t-1) parameters
**          bb = vector of b(t|t-1) parameters
**         bbb = vector of b(t) parameters
**          mu = level component of model
**           r = growth rate parameter
**
** Date: July 13, 1998
**
*/

proc (8) = pewma_filter(y,x,w,d);
    local t,n,a,aa,aaa,b,bb,bbb,mu,r;
n = rows(y);
a = zeros(n,1); aa = zeros(n,1); aaa = zeros(n,1);
b = zeros(n,1); bb = zeros(n,1); bbb = zeros(n,1);
mu = zeros(n,1); r = zeros(n,1);

t=1;
do until t>n;
if t==1;
    a[t,1] = 0;
    b[t,1] = 0;
    r[t,1] = 0;
    aa[t,1] = w.*a[t,1];    bb[t,1] = w.*b[t,1];
    aaa[t,1] = y[t,1];      bbb[t,1] = exp(x[t,.]*d + r[t,1]);
    mu[t,1] = meanc(y);

else;
    a[t,1] = aaa[t-1,1];
    b[t,1] = bbb[t-1,1];
    r[t,1] = psi(a[t,1]) - psi(w.*a[t,1]);
    aa[t,1] = w.*a[t,1];
    bb[t,1] = w.*b[t,1].*exp(-x[t,.]*d).*exp(-r[t,1]);
    aaa[t,1] = w.*a[t,1] + y[t,1];
    bbb[t,1] = w.*b[t,1] + exp(x[t,.]*d + r[t,1]);
    mu[t,1] = exp(x[t,.]*d + r[t,1]).*aaa[t,1]./bbb[t,1];
endif;
t = t+1;
endo;
retp(a,aa,aaa,b,bb,bbb,mu,r);
endp;

