/*  RNDP
**
**  Purpose:  Returns matrix of pseudo random numbers from Poisson
**            distributions with means given in input matrix
**
**  Format:   y = rndp(r,c,m)
**
**  Input:    R - scalar - number of rows in returned matrix
**            C - scalar - number of columns in returned matrix
**            M - ExE matrix of means for Poisson distribution
**                (Conformable with RxC matrix)
**
**  Output:   Y - RxC matrix containing samples from a poisson distribution
**                with the mean given in the corresponding element of the
**                input matrix
**
**  Notes:    The maximum size of input matrix for which this procedure is
**            guaranteed to work for is maxvec()./6 elements (1620 for
**            Gauss 2). Larger input matrices can be used provided the mean
**            values are not all in the range  break1 < mean < break2
**            The seed is taken from the system clock at startup but may be
**            set using the RNDSEED command ( see GAUSS command reference )
**
**  Example:  y = rndp(10,2,2~4) ;
**
**             - this gives a 10x2 matrix with column :
**                 1 being 10 samples from a poisson dist. with mean 2
**                 2 being 10 samples from a poisson dist. with mean 4
**
**  This code is by Dr. David Baird.  It is significantly faster than other
**  random Poisson generators.  The modal and normal approximations used
**  in the code are similar to those described by Press et al. (1986).
*/

proc rndp(r,c,m) ;
   local break1,break2,pmode,n,index,sample,selector,pos,work1,work2,i;
   if (r < 1);
      errorlog("Error: RNDP - number of rows must be > 0");
      retp(error(1));
   elseif (c < 1);
      errorlog("Error: RNDP - number of columns must be > 0");
      retp(error(2));
   elseif (minc(minc(m)) < 0);
      errorlog("Error: RNDP - Poisson means must be > 0"); end;
      retp(error(3));
   endif;
   break1 = 7  ;     /* Last value for which the forward search is used */
   break2 = 60 ;     /* Last value before normal approximation is used */
   m = m.*ones(r,c) ;
   m = vecr(m) ;
   n = rows(m) ;
   sample = zeros(n,1) ;
   index = seqa(1,1,n) ;
   selector = m .> break2 ;  /* Select values for normal approx. */
   pos = submat(packr(index~miss(selector,0)),0,1) ;
   if not scalmiss(pos) ;
      /* Generate sample values using normal approx */
      sample[pos,1] = int(m[pos,1]+sqrt(m[pos,1]).*rndn(rows(pos),1)+.5);
      sample[pos,1] = sample[pos,1].*(sample[pos,1] .> 0) ;
      if n == rows(pos) ; /* if no values left to be generated then exit */
         goto exit ;
      endif ;
      /* Leave only values to be processed */
      m = submat(packr(m~miss(selector,1)),0,1) ;
      index = submat(packr(index~miss(selector,1)),0,1) ;
   endif ;
   /* Select values for mode search */
   n = rows(m) ;
   pos = submat(packr(seqa(1,1,n)~miss(m .> break1,0)),0,1) ;
   if not scalmiss(pos) ;
      work1 = int(m[pos,1]) ;         /* Split mean into integer and */
      m[pos,1] = m[pos,1] - work1 ;   /* fractional parts */
      /* Table of values of cdf at mode of poisson distribution */
      let pmode[60,1] =
      0.735758882342885 0.676676416183064 0.647231888782232 0.628836935179874
      0.615960654833064 0.606302782412592 0.598713835523037 0.592547341437592
      0.587408244331942 0.583039750192986 0.579266762921713 0.575965248573065
      0.573044561348093 0.570436712827373 0.568089575608544 0.565962423009877
      0.564022911672259 0.562244986044038 0.560607389391509 0.559092584231326
      0.557685955471078 0.556375212739087 0.555149935616770 0.554001223074996
      0.552921420024416 0.551903901703405 0.550942901981355 0.550033375383794
      0.549170885281877 0.548351512577912 0.547571780589863 0.546828592844762
      0.546119181238658 0.545441062581462 0.544792001969690 0.544169981754378
      0.543573175121377 0.542999923495121 0.542448717128705 0.541918178362538
      0.541407047128553 0.540914168352479 0.540438480967288 0.539979008299820
      0.539534849632268 0.539105172772506 0.538689207493780 0.538286239726013
      0.537895606399060 0.537516690853148 0.537148918744220 0.536791754382338
      0.536444697450023 0.536107280054810 0.535779064076528 0.535459638775088
      0.535148618629082 0.534845641379320 0.534550366254748 0.534262472360959;
      i = seqa(1,1,maxc(work1)) ;
      /* Set up working matrix with columns containing :
         1 index of value in original matrix
         2 integer part of original mean
         3 poisson probability ( starting at modal value )
         4 Value of poisson cdf ( starting at modal value )
         5 Sample value for the poisson cdf                   */
      work1 = index[pos,1] ~ work1 ~ submat((exp(-i).*(i).^i./(i!)),work1,0) ~
              submat(pmode,work1,0) ~ rndu(rows(pos),1) ;
      clear pmode,pos ;
      /* Split into 2 cases - forward or backward search from mode */
      selector = work1[.,4] .< work1[.,5] ;
      work2 = packr(work1~miss(selector,1)) ;
      work1 = packr(work1~miss(selector,0)) ;
      if not scalmiss(work2) ;
         /* Backward search from mode ( stop on reaching zero ) */
         work2 = work2[.,1:5] ;
         i = work2[.,2] ;
         do until(0) ;
            work2[.,4] = work2[.,4] - work2[.,3] ; /* Update cdf */
            /* Select values satisfying cdf criterion */
            selector = (work2[.,5] .> work2[.,4]) .or (i .== 0) ;
            pos = submat(packr(seqa(1,1,rows(work2))~miss(selector,0)),0,1) ;
            if not scalmiss(pos) ;
               sample[work2[pos,1],1] = i[pos,1] ;
               if rows(work2) == rows(pos) ; /* Exit loop - All done */
                  goto modefrwd ;
               else ;
                  /* Remove values found */
                  i = submat(packr(i~miss(selector,1)),0,1) ;
                  work2 = packr(work2~miss(selector,1)) ;
               endif ;
            endif ;
            work2[.,3] = work2[.,3] .* i./work2[.,2] ; /* Update prob */
            i = i - 1 ;
         endo ;
      endif ;
modefrwd:
      if not scalmiss(work1) ;
         /* Forward search from mode */
         work1 = work1[.,1:5] ;
         i = work1[.,2] ;
         do until(0) ;
            i = i + 1 ;
            work1[.,3] = work1[.,3] .* work1[.,2]./i ; /* Update prob */
            work1[.,4] = work1[.,4] + work1[.,3] ;     /* Update cdf */
            /* Select values satisfying cdf criterion */
            selector = work1[.,5] .< work1[.,4] ;
            pos = submat(packr(seqa(1,1,rows(work1))~miss(selector,0)),0,1) ;
            if not scalmiss(pos) ;
               sample[work1[pos,1],1] = i[pos,1] ;
               if rows(work1) == rows(pos) ; /* Exit loop - All done */
                  goto forward ;
               else ;
                  /* Remove values found */
                  i = submat(packr(i~miss(selector,1)),0,1) ;
                  work1 = packr(work1~miss(selector,1)) ;
               endif ;
            endif ;
         endo ;
      endif ;
   endif ;
forward:
   /* Forward search from zero using exponential variables and transformation
      Set up working matrix  with columns containing :
         1 index of value in original matrix
         2 exp(-(original mean or fractional part of real mean if > break1))
         5 Accumulated value for the exponential variables  */
   work1 = index ~ exp(-m) ~ rndu(rows(m),1) ;
   clear m,index ;
   i = 0 ;
   do until(0) ;
      /* Select values satisfying cdf criterion */
      work1 = work1[.,1:3] ;
      selector = work1[.,2] .> work1[.,3] ;
      pos = submat(packr(seqa(1,1,rows(work1))~miss(selector,0)),0,1) ;
      if not scalmiss(pos) ;
         sample[work1[pos,1],1] = sample[work1[pos,1],1] +i.*ones(rows(pos),1);
         if rows(work1) == rows(pos) ; /* Exit - All done */
            goto exit ;
         else ;
           /* Remove values found */
           work1 = packr(work1~miss(selector,1)) ;
         endif ;
      endif ;
      /* Accumulate exponential variables */
      work1[.,3] = work1[.,3] .* rndu(rows(work1),1) ;
      i = i + 1 ;
   endo ;
exit:
   retp(reshape(sample,r,c)) ; /* Return reshaped matrix of poisson samples */
endp ;                         /* of Poisson random number generator */

